"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const HttpStatusCode_1 = require("../enums/HttpStatusCode");
const constants_1 = require("../constants/constants");
const NotFoundError_1 = require("../exceptions/NotFoundError");
const ValidationError_1 = require("../exceptions/ValidationError");
/**
 * This class represent the http response object
 */
class Response {
    /**
     * Generate success response for aws API gateway
     * @param body
     * Http response body object
     * @param statusCode
     * Http status code like 200, 204 etc
     * @return
     * Http response object as JSON string
     */
    getSuccessResponse(httpMethod, body) {
        this.setSuccessResponse(httpMethod, body);
        return this.toJSONObject();
    }
    /**
     * Generate error response for aws API gateway
     * @param error
     * Application error like 500, 400 etc
     * @return
     * Http response object as JSON string
     */
    getErrorResponse(error) {
        this.setErrorResponse(error);
        return this.toJSONObject();
    }
    /**
     * Set Response from the success body and
     * map http method to http status code
     * @param httpMethod
     * The httpMethod like GET, POST etc
     * @param body
     * Response body object
     */
    setSuccessResponse(httpMethod, body) {
        switch (httpMethod) {
            case constants_1.httpMethod.GET:
                this.body = JSON.stringify(body);
                this.statusCode = HttpStatusCode_1.HttpStatusCode.OK;
                break;
            case constants_1.httpMethod.POST:
                this.body = JSON.stringify(body);
                this.statusCode = HttpStatusCode_1.HttpStatusCode.CREATED;
                break;
            case constants_1.httpMethod.PUT:
                this.statusCode = HttpStatusCode_1.HttpStatusCode.UPDATED;
                break;
            case constants_1.httpMethod.DELETE:
                this.statusCode = HttpStatusCode_1.HttpStatusCode.UPDATED;
                break;
            default:
                throw new Error(`Unknown httpMethod ${httpMethod}`);
        }
        this.headers = { 'Access-Control-Allow-Origin': '*' };
    }
    /**
     * Set Response based on the error type and
     * map error to corresponding http status code
     * @param error
     * Application error
     */
    setErrorResponse(error) {
        let statusCode;
        let message;
        let errors;
        switch (error.constructor.name) {
            case NotFoundError_1.default.name:
                statusCode = HttpStatusCode_1.HttpStatusCode.NOT_FOUND;
                message = _.get(error, 'message');
                break;
            case ValidationError_1.default.name:
                statusCode = HttpStatusCode_1.HttpStatusCode.BAD_REQUEST;
                message = _.get(error, 'message');
                errors = _.get(error, 'errors');
                break;
            default:
                statusCode = HttpStatusCode_1.HttpStatusCode.INTERNAL_SERVER_ERROR;
                if (error instanceof Error) {
                    message = _.get(error, 'message');
                }
                else {
                    message = _.isObject(error) ? JSON.stringify(error) : error;
                }
        }
        const responseBody = {
            statusCode: statusCode,
            message: message
        };
        if (errors) {
            responseBody['errors'] = errors;
        }
        this.body = JSON.stringify(responseBody);
        this.headers = { 'Access-Control-Allow-Origin': '*' };
        this.statusCode = statusCode;
    }
    /**
     * Convert Response object instance to JSON Object
     */
    toJSONObject() {
        return JSON.parse(JSON.stringify(this));
    }
}
exports.default = Response;
//# sourceMappingURL=Response.js.map